@@ -18,17 +18,43 @@
  * @package  Net_URL2
  * @author   Some Pear Developers <pear@php.net>
  * @license  https://spdx.org/licenses/BSD-3-Clause BSD-3-Clause
- * @version  Release: 2.0.5
+ * @version  Release: 2.1.1
  * @link     https://pear.php.net/package/Net_URL2
  */
 class Net_URL2Test extends PHPUnit_Framework_TestCase
 {
     /**
-     * Tests setting an empty userinfo part
-     * Also: Regression test for Bug #20013
+     * Tests setting a zero-length string and false as authority
+     * Also: Regression test for Bug #20420
      *
+     * @covers Net_URL2::setAuthority
+     * @return void
+     * @link https://pear.php.net/bugs/bug.php?id=20420
+     */
+    public function testSetEmptyAuthority()
+    {
+        $url = new Net_URL2('http://www.example.com/');
+        $url->setAuthority('');
+        $this->assertSame('', $url->getAuthority());
+        $this->assertSame('', $url->getHost());
+        $this->assertSame(false, $url->getPort());
+        $this->assertSame(false, $url->getUserinfo());
+        $this->assertSame(false, $url->getUser());
+
+        $url->setAuthority(false);
+        $this->assertSame(false, $url->getAuthority());
+    }
+
+    /**
+     * Tests setting an empty userinfo part
+     * Also: Regression test for Bug #20013 and Bug #20399
+     *
+     * @covers Net_URL2::setUserinfo
+     * @covers Net_URL2::getUserinfo
+     * @covers Net_URL2::getURL
      * @return void
      * @link https://pear.php.net/bugs/bug.php?id=20013
+     * @link https://pear.php.net/bugs/bug.php?id=20399
      */
     public function testSetEmptyUserinfo()
     {
@@ -39,6 +65,35 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
         $this->assertSame('http://www.example.com/', $url->getURL());
         $url->setUserinfo('');
         $this->assertSame('http://www.example.com/', $url->getURL());
+        $this->assertSame('', $url->getUserinfo());
+        $url->setUserinfo(false);
+        $this->assertSame('http://www.example.com/', $url->getURL());
+        $this->assertFalse($url->getUserinfo());
+    }
+
+    /**
+     * Tests an URL with no userinfo and normalization
+     *
+     * Also: Regression test for Bug #20385
+     *
+     * @covers Net_URL2::getUserinfo
+     * @covers Net_URL2::normalize
+     * @covers Net_URL2::getNormalizedURL
+     * @return void
+     * @link https://pear.php.net/bugs/bug.php?id=20385
+     */
+    public function testNoUserinfoAndNormalize()
+    {
+        $testUrl = 'http://www.example.com/';
+
+        $url = new Net_URL2($testUrl);
+        $this->assertFalse($url->getUserinfo());
+
+        $url->normalize();
+        $this->assertFalse($url->getUserinfo());
+
+        $this->assertEquals($testUrl, $url->getURL());
+        $this->assertEquals($testUrl, $url->getNormalizedURL());
     }
 
     /**
@@ -311,7 +366,8 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
      */
     public function testUrlEncoding()
     {
-        $url = new Net_URL2('http://localhost/bug.php');
+        $options = array(Net_URL2::OPTION_DROP_SEQUENCE => false);
+        $url     = new Net_URL2('http://localhost/bug.php', $options);
         $url->setQueryVariables(
             array(
                 'indexed' => array(
@@ -367,16 +423,24 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
     /**
      * Brackets for array query variables
      *
+     * Also text to not encode zero based integer sequence into brackets
+     *
      * @return void
+     *
+     * @link https://pear.php.net/bugs/bug.php?id=20427
      */
     public function testUseBrackets()
     {
         $url = new Net_URL2('http://example.org/');
+        $url->setQueryVariables(array('foo' => array('bar', 'baz')));
+        $expected = 'http://example.org/?foo[]=bar&foo[]=baz';
+        $this->assertEquals($expected, $url->getURL());
+
+        $options = array(Net_URL2::OPTION_DROP_SEQUENCE => false);
+        $url     = new Net_URL2('http://example.org/', $options);
         $url->setQueryVariables(array('foo' => array('bar', 'foobar')));
-        $this->assertEquals(
-            'http://example.org/?foo[0]=bar&foo[1]=foobar',
-            strval($url)
-        );
+        $expected = 'http://example.org/?foo[0]=bar&foo[1]=foobar';
+        $this->assertEquals($expected, $url->getURL());
     }
 
     /**
@@ -397,6 +461,45 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
         );
     }
 
+    /**
+     * A dataProvider for example URIs from RFC 3986 Section 1.1.2
+     *
+     * @return array
+     * @link http://tools.ietf.org/html/rfc3986#section-1.1.2
+     * @see  testExampleUri
+     */
+    public function provideExampleUri()
+    {
+        return array(
+            array('ftp://ftp.is.co.za/rfc/rfc1808.txt'),
+            array('http://www.ietf.org/rfc/rfc2396.txt'),
+            array('ldap://[2001:db8::7]/c=GB?objectClass?one'),
+            array('mailto:John.Doe@example.com'),
+            array('news:comp.infosystems.www.servers.unix'),
+            array('tel:+1-816-555-1212'),
+            array('telnet://192.0.2.16:80/'),
+            array('urn:oasis:names:specification:docbook:dtd:xml:4.1.2'),
+        );
+    }
+
+    /**
+     * test that Net_URL2 works with the example URIs from RFC 3986 Section 1.1.2
+     *
+     * @param string $uri example URI
+     *
+     * @return       void
+     * @dataProvider provideExampleUri
+     * @link         http://tools.ietf.org/html/rfc3986#section-1.1.2
+     * @see          testComponentRecompositionAndNormalization
+     */
+    public function testExampleUri($uri)
+    {
+        $url = new Net_URL2($uri);
+        $this->assertSame($uri, $url->__toString());
+        $url->normalize();
+        $this->assertSame($uri, $url->__toString());
+    }
+
     /**
      * A dataProvider for pairs of paths with dot segments and
      * their form when removed.
@@ -554,6 +657,56 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
         $this->assertSame($expected, $actual);
     }
 
+    /**
+     * data provider of host and port
+     *
+     * @return array
+     * @see testHostAndPort
+     */
+    public function provideHostAndPort()
+    {
+        return array(
+            array('[::1]', '[::1]', false),
+            array('[::1]:', '[::1]', false),
+            array('[::1]:128', '[::1]', '128'),
+            array('127.0.0.1', '127.0.0.1', false),
+            array('127.0.0.1:', '127.0.0.1', false),
+            array('127.0.0.1:128', '127.0.0.1', '128'),
+            array('localhost', 'localhost', false),
+            array('localhost:', 'localhost', false),
+            array('localhost:128', 'localhost', '128'),
+        );
+    }
+
+    /**
+     * test that an authority containing host and port maps to expected host and port
+     *
+     * This is also a regression test to test that using ip-literals works along-
+     * side ipv4 and reg-name hosts incl. port numbers
+     *
+     * It was reported as Bug #20423 on 2014-10-06 18:25 UTC that
+     * http://[::1]// URI drops the host
+     *
+     * @param string      $authority    string
+     * @param string      $expectedHost string
+     * @param string|bool $expectedPort string or FALSE
+     *
+     * @return void
+     * @dataProvider provideHostAndPort
+     * @covers       Net_URL2::setAuthority()
+     * @link         https://pear.php.net/bugs/bug.php?id=20423
+     * @link         http://tools.ietf.org/html/rfc3986#section-3.2.2
+     * @link         http://tools.ietf.org/html/rfc3986#section-3.2
+     * @link         http://tools.ietf.org/html/rfc3986#section-3.2.3
+     */
+    public function testHostAndPort($authority, $expectedHost, $expectedPort)
+    {
+        $uri = "http://{$authority}";
+        $url = new Net_URL2($uri);
+        $this->assertSame($expectedHost, $url->getHost());
+        $this->assertSame($expectedPort, $url->getPort());
+    }
+
     /**
      * This is a regression test to test that Net_URL2::getQueryVariables() does
      * not have a problem with nested array values in form of stacked brackets and
@@ -686,6 +839,9 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
 
             // Bug #20161: URLs with "0" as host fail to normalize with empty path
             array('http://0/', 'http://0'),
+
+            // Bug #20473: Normalize query and fragment broken
+            array('foo:///?%66%6f%6f#%62%61%72', 'foo:///?foo#bar'),
         );
     }
 
@@ -735,6 +891,49 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
         $this->assertEquals(false, $url->$property);
     }
 
+    /**
+     * data provider of uri and normal URIs
+     *
+     * @return array
+     * @see testComponentRecompositionAndNormalization
+     */
+    public function provideComposedAndNormalized()
+    {
+        return array(
+            array(''),
+            array('http:g'),
+            array('user@host'),
+            array('mailto:user@host'),
+        );
+    }
+
+    /**
+     * Tests Net_URL2 RFC 3986 5.3. Component Recomposition in the light
+     * of normalization
+     *
+     * This is also a regression test to test that a missing authority works well
+     * with normalization
+     *
+     * It was reported as Bug #20418 on 2014-10-02 22:10 UTC that there is an
+     * Incorrect normalization of URI with missing authority
+     *
+     * @param string $uri URI
+     *
+     * @return       void
+     * @covers       Net_URL2::getUrl()
+     * @covers       Net_URL2::normalize()
+     * @dataProvider provideComposedAndNormalized
+     * @link         https://pear.php.net/bugs/bug.php?id=20418
+     * @see          testExampleUri
+     */
+    public function testComponentRecompositionAndNormalization($uri)
+    {
+        $url = new Net_URL2($uri);
+        $this->assertSame($uri, $url->getURL());
+        $url->normalize();
+        $this->assertSame($uri, $url->getURL());
+    }
+
     /**
      * Tests Net_URL2 ctors URL parameter works with objects implementing
      * __toString().
@@ -853,4 +1052,37 @@ class Net_URL2Test extends PHPUnit_Framework_TestCase
         $url->setHost('example.com');
         $this->assertSame('//example.com/index.html', (string)$url);
     }
+
+    /**
+     * This is a regression test to test that using the file:// URI scheme with
+     * an empty (default) hostname has the empty authority preserved when the
+     * full URL is build.
+     *
+     * It was reported as Bug #20304 on 2014-06-21 00:06 UTC
+     * that file:// URI are crippled.
+     *
+     * Tests with a default authority for the "file" URI scheme
+     *
+     * @covers Net_URL2::getURL
+     * @return void
+     * @link https://pear.php.net/bugs/bug.php?id=20304
+     */
+    public function test20304()
+    {
+        $file = 'file:///path/to/file';
+        $url = new Net_URL2($file);
+        $this->assertSame($file, (string) $url);
+
+        $file = 'file://localhost/path/to/file';
+        $url = new Net_URL2($file);
+        $this->assertSame($file, (string) $url);
+
+        $file = 'file://user@/path/to/file';
+        $url = new Net_URL2($file);
+        $this->assertSame($file, (string) $url);
+
+        $file = 'FILE:///path/to/file';
+        $url = new Net_URL2($file);
+        $this->assertSame($file, (string) $url);
+    }
 }
